
use strict;
use warnings;

use Test::More;

BEGIN { use_ok('Regexp::SAR') };


#{
#  my $rootNode = Regexp::SAR::buildRootNode();
#  is(Regexp::SAR::getCharsNumber($rootNode), 0, "Number of characters in node");
#  is(Regexp::SAR::getCharsAsStr($rootNode), "", "characters in node");
#}





##############################################
done_testing();
